searchNodes=[{"doc":"ExArray.Array is a module that provides Array-ish functionality to Elixir. The ExArray.Array structure is useful when fast random access, by index, is needed. It can be dynamically grown and shrunk by removing or adding elements from any position in the structure. When passing indexes to any function in the module, negative integers are supported. Negative integers are treated as an offset from the end of the structure. For example, if the structure contains 5 elements, the index -1 will return the last element, -2 will return the second to last element, and so on. If the negative index runs over the beginning of the structure, the appropriate failure condition will occur, either raises or an error tuple, depending on the method being called. In the example with 5 elements, the index -6 will result in an error, as there is no element before the first element. When a positive index is passed, the index is treated as an offset from the beginning and the appropriate failure condition will occur if the index runs over the end of the structure. Construction ExArray.Array can be constructed in a number of ways. The most common is to use the new/0 function to create an empty structure. Alternatively, you can use the new/1 function to create a structure containing the elements in the given list. iex&gt; ExArray.Array . new ( ) % ExArray.Array { length : 0 , contents : %{ } } iex&gt; ExArray.Array . new ( [ 1 , 2 , 3 ] ) % ExArray.Array { length : 3 , contents : %{ 0 =&gt; 1 , 1 =&gt; 2 , 2 =&gt; 3 } } Accessing elements Elements can be accessed by index using either the get/2 or get!/2 functions. The difference between these two functions is that get!/2 will raise an error if the index is out of bounds, while get/2 will return an error tuple. iex&gt; ExArray.Array . get ( ExArray.Array . new ( [ 1 , 2 , 3 ] ) , 1 ) { :ok , 2 } iex&gt; ExArray.Array . get ( ExArray.Array . new ( [ 1 , 2 , 3 ] ) , 3 ) { :error , :out_of_bounds } iex&gt; ExArray.Array . get! ( ExArray.Array . new ( [ 1 , 2 , 3 ] ) , 1 ) 2 iex&gt; ExArray.Array . get! ( ExArray.Array . new ( [ 1 , 2 , 3 ] ) , 3 ) ** (ArgumentError) Index 3 is out of bounds for length 3 Adding Elements Adding elements to the end of the structure is fast as no other elements need to be re-indexed. This is done using the add/2 function. The add/2 function returns a new structure with the element added to the end. This operation should always succeed iex&gt; ExArray.Array . add ( ExArray.Array . new ( [ 1 , 2 , 3 ] ) , 4 ) % ExArray.Array { length : 4 , contents : %{ 0 =&gt; 1 , 1 =&gt; 2 , 2 =&gt; 3 , 3 =&gt; 4 } } Adding elements to the beginning or middle of the structure is slower as all elements after the index need to be re-indexed. This is done using add_at/3 or add_at!/3 . These functions return a new structure with the element added at the given index. If an element existed at the given index, it and all other elements after it will be shifted one to the right. iex&gt; ExArray.Array . add_at ( ExArray.Array . new ( [ 1 , 2 , 3 ] ) , 1 , 4 ) { :ok , % ExArray.Array { length : 4 , contents : %{ 0 =&gt; 1 , 1 =&gt; 4 , 2 =&gt; 2 , 3 =&gt; 3 } } } iex&gt; ExArray.Array . add_at ( ExArray.Array . new ( [ 1 , 2 , 3 ] ) , 4 , 4 ) { :error , :out_of_bounds } iex&gt; ExArray.Array . add_at! ( ExArray.Array . new ( [ 1 , 2 , 3 ] ) , 1 , 4 ) % ExArray.Array { length : 4 , contents : %{ 0 =&gt; 1 , 1 =&gt; 4 , 2 =&gt; 2 , 3 =&gt; 3 } } iex&gt; ExArray.Array . add_at! ( ExArray.Array . new ( [ 1 , 2 , 3 ] ) , 4 , 4 ) ** (ArgumentError) Index 4 is out of bounds for length 3 Removing Elements Removing elements from the end of the structure is fast as no other elements need to be re-indexed. This is done using the remove/1 function. The remove/1 function returns a new structure with the last element removed. If the ExArray.Array is empty, the remove/1 function will return the original structure. This operation should always succeed iex&gt; ExArray.Array . remove ( ExArray.Array . new ( [ 1 , 2 , 3 ] ) ) % ExArray.Array { length : 2 , contents : %{ 0 =&gt; 1 , 1 =&gt; 2 } } iex&gt; ExArray.Array . remove ( ExArray.Array . new ( ) ) % ExArray.Array { length : 0 , contents : %{ } } Removing elements from the beginning or middle of the structure is slower as all elements after the index need to be re-indexed. This is done using remove_at/2 or remove_at!/2 . These functions return a new structure with the element at the given index removed. All elements after it will be shifted one to the left. iex&gt; ExArray.Array . remove_at ( ExArray.Array . new ( [ 1 , 2 , 3 ] ) , 1 ) { :ok , % ExArray.Array { length : 2 , contents : %{ 0 =&gt; 1 , 1 =&gt; 3 } } } iex&gt; ExArray.Array . remove_at ( ExArray.Array . new ( [ 1 , 2 , 3 ] ) , 4 ) { :error , :out_of_bounds } iex&gt; ExArray.Array . remove_at! ( ExArray.Array . new ( [ 1 , 2 , 3 ] ) , 1 ) % ExArray.Array { length : 2 , contents : %{ 0 =&gt; 1 , 1 =&gt; 3 } } iex&gt; ExArray.Array . remove_at! ( ExArray.Array . new ( [ 1 , 2 , 3 ] ) , 4 ) ** (ArgumentError) Index 4 is out of bounds for length 3 Collectable Operations ExArray.Array implements the Collectable protocol. This means that it can be used with Enum.into and for special forms iex&gt; Enum . into ( [ 1 , 2 , 3 ] , ExArray.Array . new ( ) ) % ExArray.Array { length : 3 , contents : %{ 0 =&gt; 1 , 1 =&gt; 2 , 2 =&gt; 3 } } iex&gt; for x &lt;- [ 1 , 2 , 3 ] , into : ExArray.Array . new ( ) , do : x % ExArray.Array { length : 3 , contents : %{ 0 =&gt; 1 , 1 =&gt; 2 , 2 =&gt; 3 } } Enum Operations ExArray.Array implements the Enumerable protocol. This means that it can be used with the functions in the Enum module. iex&gt; ExArray.Array . new ( [ 1 , 2 , 3 ] ) |&gt; Enum . map ( &amp; ( &amp;1 * 2 ) ) [ 2 , 4 , 6 ] iex&gt; ExArray.Array . new ( [ 1 , 2 , 3 ] ) |&gt; Enum . reduce ( &amp; ( &amp;1 + &amp;2 ) ) 6 iex&gt; ExArray.Array . new ( [ 1 , 2 , 3 ] ) |&gt; Enum . filter ( &amp; ( &amp;1 &gt; 1 ) ) [ 2 , 3 ] iex&gt; ExArray.Array . new ( [ 1 , 2 , 3 ] ) |&gt; Enum . at ( 1 ) 2","ref":"ExArray.html","title":"ExArray","type":"module"},{"doc":"An ExArray.Array is a data structure that contains a collection of elements. Elements are stored in a Map with their index as the key. This allows for fast random access to elements by index, but slower insertion and deletion operations, unless the element is being added or deleted to the end of the Array .","ref":"ExArray.Array.html","title":"ExArray.Array","type":"module"},{"doc":"Adds the given element to the end of the Array Examples iex&gt; ExArray.Array . new ( [ 1 , 2 , 3 ] ) |&gt; ExArray.Array . add ( 4 ) % ExArray.Array { length : 4 , contents : %{ 0 =&gt; 1 , 1 =&gt; 2 , 2 =&gt; 3 , 3 =&gt; 4 } }","ref":"ExArray.Array.html#add/2","title":"ExArray.Array.add/2","type":"function"},{"doc":"Adds the given element to the Array at the given index. If the given index is greater than the length of the Array, an {:error, :out_of_bounds} tuple is returned. If the given index is negative, it will be evaluated as an offset from the end of the Array. If an element currently exists at the given index, it, and all following elements, are shifted right. Examples iex&gt; ExArray.Array . new ( [ 1 , 2 , 3 ] ) |&gt; ExArray.Array . add_at ( 1 , 4 ) { :ok , % ExArray.Array { length : 4 , contents : %{ 0 =&gt; 1 , 1 =&gt; 4 , 2 =&gt; 2 , 3 =&gt; 3 } } } iex&gt; ExArray.Array . new ( [ 1 , 2 , 3 ] ) |&gt; ExArray.Array . add_at ( - 2 , 4 ) { :ok , % ExArray.Array { length : 4 , contents : %{ 0 =&gt; 1 , 1 =&gt; 4 , 2 =&gt; 2 , 3 =&gt; 3 } } } iex&gt; ExArray.Array . new ( [ 1 , 2 , 3 ] ) |&gt; ExArray.Array . add_at ( 4 , 4 ) { :error , :out_of_bounds }","ref":"ExArray.Array.html#add_at/3","title":"ExArray.Array.add_at/3","type":"function"},{"doc":"Adds the given element to the Array at the given index. If the given index is greater than the length of the Array, an ArgumentError is raised. If the given index is negative, it will be evaluated as an offset from the end of the Array. If an element currently exists at the given index, it, and all following elements, are shifted right.t() Examples iex&gt; ExArray.Array . new ( [ 1 , 2 , 3 ] ) |&gt; ExArray.Array . add_at! ( 1 , 4 ) % ExArray.Array { length : 4 , contents : %{ 0 =&gt; 1 , 1 =&gt; 4 , 2 =&gt; 2 , 3 =&gt; 3 } } iex&gt; ExArray.Array . new ( [ 1 , 2 , 3 ] ) |&gt; ExArray.Array . add_at! ( - 2 , 4 ) % ExArray.Array { length : 4 , contents : %{ 0 =&gt; 1 , 1 =&gt; 4 , 2 =&gt; 2 , 3 =&gt; 3 } } iex&gt; ExArray.Array . new ( [ 1 , 2 , 3 ] ) |&gt; ExArray.Array . add_at! ( 4 , 4 ) ** (ArgumentError) Index 4 is out of bounds for length 3","ref":"ExArray.Array.html#add_at!/3","title":"ExArray.Array.add_at!/3","type":"function"},{"doc":"Returns {:ok, element} if the given index is within the bounds of the Array, otherwise returns {:error, :out_of_bounds} . If the given index is negative, it will be evaluated as an offset from the end of the Array. Examples iex&gt; ExArray.Array . new ( [ 1 , 2 , 3 ] ) |&gt; ExArray.Array . get ( 1 ) { :ok , 2 } iex&gt; ExArray.Array . new ( [ 1 , 2 , 3 ] ) |&gt; ExArray.Array . get ( - 1 ) { :ok , 3 } iex&gt; ExArray.Array . new ( [ 1 , 2 , 3 ] ) |&gt; ExArray.Array . get ( 4 ) { :error , :out_of_bounds }","ref":"ExArray.Array.html#get/2","title":"ExArray.Array.get/2","type":"function"},{"doc":"Returns the element at the given index if it exists, otherwise raises an ArgumentError If the given index is negative, it will be evaluated as an offset from the end of the Array. Examples iex&gt; ExArray.Array . new ( [ 1 , 2 , 3 ] ) |&gt; ExArray.Array . get! ( 1 ) 2 iex&gt; ExArray.Array . new ( [ 1 , 2 , 3 ] ) |&gt; ExArray.Array . get! ( - 1 ) 3 iex&gt; ExArray.Array . new ( [ 1 , 2 , 3 ] ) |&gt; ExArray.Array . get! ( 4 ) ** (ArgumentError) Index 4 is out of bounds for length 3","ref":"ExArray.Array.html#get!/2","title":"ExArray.Array.get!/2","type":"function"},{"doc":"Creates an ExArray.Array with a length of zero, containing no elements. Examples iex&gt; ExArray.Array . new ( ) % ExArray.Array { length : 0 , contents : %{ } }","ref":"ExArray.Array.html#new/0","title":"ExArray.Array.new/0","type":"function"},{"doc":"Creates an ExArray.Array with the given list of elements. Examples iex&gt; ExArray.Array . new ( [ 1 , 2 , 3 ] ) % ExArray.Array { length : 3 , contents : %{ 0 =&gt; 1 , 1 =&gt; 2 , 2 =&gt; 3 } }","ref":"ExArray.Array.html#new/1","title":"ExArray.Array.new/1","type":"function"},{"doc":"Removes the last element from the Array and returns the resulting Array. If the array is empty, the original Array is returned. Examples iex&gt; ExArray.Array . new ( [ 1 , 2 , 3 ] ) |&gt; ExArray.Array . remove ( ) % ExArray.Array { length : 2 , contents : %{ 0 =&gt; 1 , 1 =&gt; 2 } } iex&gt; ExArray.Array . new ( [ ] ) |&gt; ExArray.Array . remove ( ) % ExArray.Array { length : 0 , contents : %{ } }","ref":"ExArray.Array.html#remove/1","title":"ExArray.Array.remove/1","type":"function"},{"doc":"Removes the element at the given index from the Array and returns the resulting Array. If the given index is invalid, an {:error, :out_of_bounds} tuple is returned. If the given index is negative, it will be evaluated as an offset from the end of the Array. Examples iex&gt; ExArray.Array . new ( [ 1 , 2 , 3 ] ) |&gt; ExArray.Array . remove_at ( 1 ) { :ok , % ExArray.Array { length : 2 , contents : %{ 0 =&gt; 1 , 1 =&gt; 3 } } } iex&gt; ExArray.Array . new ( [ 1 , 2 , 3 ] ) |&gt; ExArray.Array . remove_at ( - 1 ) { :ok , % ExArray.Array { length : 2 , contents : %{ 0 =&gt; 1 , 1 =&gt; 2 } } } iex&gt; ExArray.Array . new ( [ 1 , 2 , 3 ] ) |&gt; ExArray.Array . remove_at ( 4 ) { :error , :out_of_bounds }","ref":"ExArray.Array.html#remove_at/2","title":"ExArray.Array.remove_at/2","type":"function"},{"doc":"Removes the element at the given index from the Array and returns the resulting Array. If the given index is invalid, an ArgumentError is raised. If the given index is negative, it will be evaluated as an offset from the end of the Array. Examples iex&gt; ExArray.Array . new ( [ 1 , 2 , 3 ] ) |&gt; ExArray.Array . remove_at! ( 1 ) % ExArray.Array { length : 2 , contents : %{ 0 =&gt; 1 , 1 =&gt; 3 } } iex&gt; ExArray.Array . new ( [ 1 , 2 , 3 ] ) |&gt; ExArray.Array . remove_at! ( - 1 ) % ExArray.Array { length : 2 , contents : %{ 0 =&gt; 1 , 1 =&gt; 2 } } iex&gt; ExArray.Array . new ( [ 1 , 2 , 3 ] ) |&gt; ExArray.Array . remove_at! ( 4 ) ** (ArgumentError) Index 4 is out of bounds for length 3","ref":"ExArray.Array.html#remove_at!/2","title":"ExArray.Array.remove_at!/2","type":"function"},{"doc":"Returns a List of all the elements in the Array. Ordering is retained. Examples iex&gt; ExArray.Array . new ( [ 1 , 2 , 3 ] ) |&gt; ExArray.Array . to_list ( ) [ 1 , 2 , 3 ]","ref":"ExArray.Array.html#to_list/1","title":"ExArray.Array.to_list/1","type":"function"},{"doc":"","ref":"ExArray.Array.html#t:t/0","title":"ExArray.Array.t/0","type":"type"}]